<?php

class MKVTagger
{
	const MIME = 'video/x-matroska';
	const CHARSET = 'UTF-8';
	
	public $Tags = array();
	
	public static function __New()
	{
		return new self();
	}
	
	public function __construct()
	{
		return $this;
	}
	
	public function setReleaseDate($rel)
	{
		$this->Tags['DATE_RELEASED'] = $rel;
		return $this;
	}
	
	public function fetchFromTMDB($TMDB)
	{
		if($TMDB instanceof TMDBMovie)
		{
			$this	->setTitle	($TMDB->title)
				->setReleaseDate($TMDB->release_date)
				->setOriginal_title($TMDB->original_title)
				->setSummary	($TMDB->overview)
				->setDescription($TMDB->overview)
				->setSynopsis	($TMDB->tagline)
				->setGenre	($TMDB->implodeGenres())
				->setContent_Type	('Film')
				->setProduction_Studio	($TMDB->implode('production_companies'))
				->setRecording_Location	($TMDB->implodeProductionCountries())
			;
		}
		elseif($TMDB instanceof TMDBTvSeasonEpisode)
		{
			$this	->setTitle	($TMDB->name)
// 				->setSummary	($TMDB->overview)
				->setDescription($TMDB->overview)
				->setSynopsis	($TMDB->overview)
// 				->setComments	($TMDB->overview)
// 				->setComment	($TMDB->overview)
				->setReleaseDate($TMDB->air_date)
				->setSeason	($TMDB->season_number)
				->setEpisode	($TMDB->episode_number)
				->setContent_Type('TV Show')
			;
		}
		elseif(is_array($TMDB))
			throw new Exception('You need to pass a valid TMDB Object (such as TMDBMovie or TMDBTvSeasonEpisode) to MKVTagger::fetchFromTMDB.');
		else
			throw new Exception('Unknown error occured at MKVTagger::fetchFromTMDB.');
		
		return $this;
	}
	
	public function __call($name, $args)
	{
		$what = substr($name, 3);
		switch(substr($name, 0, 3))
		{
			default:
			case 'set':
				$this->Tags[$what] = $args[0];
				return $this;
			break;
			case 'get':
				return $this->Tags[$what];
			break;
// 			default:
// 				call_user_func(array(&$this, $name), $args);
// 			break;
		}
	}
	
	public function __set($name, $value)
	{
		$this->Tags[$name] = $value;
		return $this;
	}
	
	public function __get($name)
	{
		return $this->Tags[$name];
	}
	
	public function toXML($file = null)
	{
		$this->DATE_TAGGED = date('Y-m-d');
		$xml = new XMLWriter();
		$xml->openMemory();
		$xml->setIndent(true);
		$xml->setIndentString("\t");
		$xml->startDocument('1.0', self::CHARSET);
		$xml->startDTD('Tags', null, 'matroskatags.dtd');
		$xml->endDTD();
		$xml->startElement('Tags');
		$xml->startElement('Tag');
		foreach($this->Tags as $tag => $value)
		{
			$xml->startElement('Simple');
				$xml->startElement('Name');
					$xml->text(strtoupper($tag));
				$xml->endElement();
				$xml->startElement('String');
					$xml->text($value);
				$xml->endElement();
			$xml->endElement();
		}
		$xml->endElement();
		$xml->endElement();
		$xml->endDocument();
		if(!is_null($file))
			file_put_contents($file, $xml->outputMemory());
		else
			return $xml->outputMemory();
		return $this;	
	}

	public function insertInto($file)
	{
		$finfo = new finfo(FILEINFO_MIME_TYPE);
		if($finfo->file($file) != self::MIME)
			throw new Exception ('File is not a Matroska.');
		$this->toXML($xml = tempnam(sys_get_temp_dir(), 'MKVTAGGER_XML_'));
		$mkvpropedit = new MKVPropEdit($file);
		$mkvpropedit->AddTags($xml)->Exec();
		unlink($xml);
		return true;
	}
	// alias of insertInto();
	public function toFile($file)
	{
		return $this->insertInto($file);
	}
}

abstract class MKVCLI
{
	protected $file = '';
	protected $args = '';
	protected $executed = false;
	
	public static $AcceptedCoverMime = array(
		'image/jpg' => 'jpg',
		'image/jpeg'=> 'jpg',
		'image/png' => 'png'
		);
	
	protected function _($r)
	{
		$this->args .= $r.' ';
		return $this;
	}
}

class MKVPropEdit extends MKVCLI
{	

	const EXEC_ON_DESTRUCT = false;
	
	public function __construct($file)
	{
		$this->file = $file;
		return $this;
	}
	
	public function __destruct()
	{
		if(self::EXEC_ON_DESTRUCT)
			$this->Exec();
		return;
	}
	
	public function Exec()
	{
		if($this->executed)
			return false;
		exec((string)$this); // call __toString()
		return $this->executed = true;
	}
	
	public function __toString()
	{
		return 'mkvpropedit "'
			.$this->file.'" '
			.$this->args;
	}
	
	public function AddTags($xmlfile)
	{
		if($xmlfile instanceof MKVTagger)
			$xmlfile->toXML($file = tempnam(sys_get_temp_dir(), 'MKVPROPEDIT_'));
		else
			$file = $xmlfile;
		$this->_('--tags "global:'.$file.'"');
		return $this;
	}
	
	public function AddCover($imgfile)
	{
		$fi = new finfo(FILEINFO_MIME_TYPE);
		if(!in_array($mime = $fi->file($imgfile), array_keys(parent::$AcceptedCoverMime)))
			throw new Exception('Unaccepted file type '.$imgfile.' ['.$mime.']');
		$this	->_('--attachment-description cover')
			->_('--attachment-name cover.'.parent::$AcceptedCoverMime[$mime])
			->_('--add-attachment "'.$imgfile.'"');
		return $this;
	}

}

class MKVMerge extends MKVCLI
{
	private $options = array();
	protected $tomerge = array();
	public function __construct($outfile)
	{
		if(substr($outfile, -4) != '.mkv')
			$outfile .= '.mkv';
		$this->file = $outfile;
	}
	
	public function Title($title)
	{
		$this->options[] = '--title "'.$title.'"';
		return $this;
	}
	
	public function addChapter($chapfile)
	{
		$this->options[] = '--chapters "'.$chapfile.'"';
		return $this;
	}
	
	//alias of addCover
	public function addPoster($imgfile)
	{
		return $this->addCover($imgfile);
	}
	
	public function addCover($imgfile)
	{
		$fi = new finfo(FILEINFO_MIME_TYPE);
		if(!in_array($mime = $fi->file($imgfile), array_keys(parent::$AcceptedCoverMime)))
			throw new Exception('Unaccepted file type '.$imgfile.' ['.$mime.']');
		$this->options[] = '--attachment-description cover';
		$this->options[] = '--attachment-name cover.'.parent::$AcceptedCoverMime[$mime];
		$this->options[] = '--attach-file "'.$imgfile.'"';
		return $this;
	}
	
	public function addTags($mkvtag)
	{
		if($mkvtag instanceof MKVTagger)
			$mkvtag->toXML($file = tempnam(sys_get_temp_dir(), 'MKVMERGE_'));
		else
			$file = $mkvtag;
		$this->options[] = '--global-tags "'.$file.'"';
		return $this;
	}
	
	public function addTrack(MKVMergeTrack $track)
	{
		$this->tomerge[] = (string) $track;
		return $this;
	}
	
	public function Exec()
	{
		if($this->executed)
			return false;
		exec((string)$this);
		return $this->executed = true;	
	}
	
	public function __toString()
	{
		$this->args = 
			implode(' ', $this->options) .
			' ' .
			implode(' ', $this->tomerge);
		return 'mkvmerge -o "'
			.$this->file.'" '
			.$this->args;
	}
}

class MKVMergeTrack extends MKVMerge
{
	private $t_opt = array();
	private $track = '';
	
	static function __New($file)
	{
		return new self($file);
	}
	
	public function __construct($file)
	{
		$this->track = $file;
		return $this;
	}
	
	public function opt($key, $value = 1, $tid = 0)
	{
		//default values
		if(is_null($tid)) 	$tid = 0;
		if(is_null($value)) 	$value = 1;
		//func
		$this->t_opt[] =
			'--'.strtolower($key).' '.
			$tid.':'.
			$value;
		return $this;
	}
	
	public function no($key)
	{
		if(!substr($key, 3) == 'no-')
			$key = 'no-'.$key;
		$this->t_opt[] = '--'.strtolower($key);
		return $this;
	}
	
	public function select_tracks($key, $value)
	{
		if(!substr($key, -7) == '-tracks')
			$key .= '-tracks';
		$this->t_opt[] = '--'.$key.' '.
			$value;
		return $this;
	}
	
	public function __call($name, $args)
	{
		$key = preg_replace('/(?<!^)([A-Z])/', '-\\1', $name);
		$value = isset($args[0]) ? $args[0] : null;
		$tid = isset($args[1]) ? $args[1] : null;
		if(substr($key, 0, 3)=='no-') /// no-audio, no-subtitle kinda commands...
			return $this->no($key);
		elseif(substr($key, -7) == '-tracks') // audio-tracks, video-tracks...
			return $this->select_track($key, $value);
		else
			return $this->opt($key, $value, $tid);
		return $this;
	}
	
	public function __toString()
	{
		return implode(' ', $this->t_opt).
			' '.
			'"'.$this->track.'"';
	}
}

?>
